//
//  PTRCalculatedLocation.h
//  Pointr
//
//  Created by Erkan Sen on 27.12.2017.
//

#import <PointrKit/PTRLocation.h>

#import <PointrKit/PTRPositioningTypes.h>

/// Location model for locations calculated via positioning
/// In addition to fields in Location, includes type, state, orientation, timestamp and some accuracy metrics
OBJC_EXPORT @interface PTRCalculatedLocation : PTRLocation

- (instancetype)init NS_UNAVAILABLE;

/// Default initialiser
/// @param coordinate Location coordinate
/// @param level Location's level
/// @param type Location type
/// @param state Location state
/// @param accuracy Confidence radius, in meters
/// @param heading Heading in radians
/// @param headingAccuracy Heading accuracy spread in radians
/// @param headingAccuracyClass Heading accuracy class
- (instancetype)initWithCoordinate:(CLLocationCoordinate2D)coordinate
                             level:(PTRLevel *)level
                              type:(PTRLocationType)type
                             state:(PTRLocationState)state
                          accuracy:(double)accuracy
                           heading:(double)heading
                   headingAccuracy:(double)headingAccuracy
              headingAccuracyClass:(PTRHeadingAccuracyClass)headingAccuracyClass;

/// Initialiser for invalid location.
+ (instancetype)invalidLocation;

/**
 Checks validity of heading properties.
 @return return valid if no field has an invalid value
 */
- (BOOL)isHeadingValid;

/**
 * Getter for type of location (Indoor, Outdoor, Transition or Unknown)
 * @note Returns Unknown for invalid locations
 * @return Location type
 */
@property (nonatomic, readonly) PTRLocationType type;

/**
 * Getter for state of location (Lost, Faded, Active or Unknown)
 * @note Returns Unknown for invalid locations
 * @return Location state
 */
@property (nonatomic, readonly) PTRLocationState state;

/**
 * Getter for estimated positional accuracy of location
 * @return Accuracy in meters
 */
@property (nonatomic, readonly) double accuracy;

/**
 * Getter for heading of location relative to true north
 * @note Returns INVALID_FLOAT when user location is not calibrated or device sensors are faulty
 * @return Heading in radians
 */
@property (nonatomic, readonly) double heading;

/**
 * Getter for estimated accuracy of heading (potential spread error)
 * @return Heading accuracy in radians
 */
@property (nonatomic, readonly) double headingAccuracy;

/**
 * Getter for accuracy class of heading (Low or High)
 * Low is for locations before convergence or for outdoor locations, High is for locations after convergence
 * @note High heading accuracy class is only possible for devices with a gyroscope
 * @return Heading accuracy class
 */
@property (nonatomic, readonly) PTRHeadingAccuracyClass headingAccuracyClass;

/**
 * Getter for calculation time of location
 * @return Location calculation timestamp
 */
@property (nonatomic, readonly) double timestamp;

@end
