//
//  PTRDataManagerInterface.h
//  Pointr
//
//  Created by Pointr on 18/04/2017.
//  Copyright © 2017 Pointr. All rights reserved.
//

#import <Foundation/Foundation.h>

// Management
#import <PointrKit/PTRListenerHelper.h>

// Data
#import <PointrKit/PTRDataTypes.h>
#import <PointrKit/PTRSite.h>

NS_ASSUME_NONNULL_BEGIN

/// Data Manager delegate.
OBJC_EXPORT @protocol PTRDataManagerDelegate<NSObject>

@optional

/**
 *  Notifies listeners when all data needed for displaying a site is available
 *  @warn Note this doesn't mean data is latest. It only means there is valid data to show.
 *  @warn Both raster maps and tile maps are considered for this check.
 *
 *  @param site          Site where this data belongs to
 */
- (void)onDataManagerReadyForSite:(nonnull PTRSite *)site;

/**
 *  Notifies listeners when Data Manager starts data management
 *
 *  @param site          Site on which data management did start. If data type is global and not site specific, the
 * site won't be valid. @see method isValid
 *  @param isOnlineData     Whether data is from server (online) or local bundle (offline)
 */
- (void)onDataManagerStartDataManagementForSite:(nonnull PTRSite *)site dataFromOnline:(BOOL)isOnlineData;

/**
 * Notifies listeners when all data processing (both offline and online) have been completed.
 *
 *  @param site          Site on which data management did complete. If data type is global and not site specific,
 * the site won't be valid. @see method isValid
 *  @param isSuccessful     Whether all updates have succeeeded or not
 *  @param isOnlineData     Whether data is from server (online) or local bundle (offline)
 *  @param errorMessages    (Nullable) A dictionary containing error messages (if any), mapped by data type
 */
- (void)onDataManagerCompleteAllForSite:(nonnull PTRSite *)site
                           isSuccessful:(BOOL)isSuccessful
                         dataFromOnline:(BOOL)isOnlineData
                          errorMessages:(nonnull NSDictionary<NSNumber *, NSString *> *)errorMessages;

/**
 *  Notifies listeners when a specific type of data (either offline or online) has started to be processed
 *
 *  @param site          Site on which data management did begin processing. If data type is global and not site
 * specific, the site won't be valid. @see method isValid
 *  @param dataType         Type of data
 *  @param isOnlineData     Whether data is from server (online) or local bundle (offline)
 */
- (void)onDataManagerBeginProcessingDataForSite:(nonnull PTRSite *)site
                                       dataType:(PTRDataType)dataType
                                 dataFromOnline:(BOOL)isOnlineData;

/**
 *  Notifies listeners when a specific type of data (either offline or online) has completed processing (whether
 * successful or not)
 *
 *  @param site          Site on which data management did end processing. If data type is global and not site
 * specific, the site won't be valid. @see method isValid
 *  @param dataType         Type of data
 *  @param isOnlineData     Whether data is from server (online) or local bundle (offline)
 *  @param isSuccessful     Whether update has succeeeded or not
 *  @param errorMessages    (Nullable) A list of error message (if any)
 */
- (void)onDataManagerEndProcessingDataForSite:(nonnull PTRSite *)site
                                     dataType:(PTRDataType)dataType
                               dataFromOnline:(BOOL)isOnlineData
                                 isSuccessful:(BOOL)isSuccessful
                                errorMessages:(nonnull NSArray<NSString *> *)errorMessages;

@end

/**
 *  Data Manager is responsible for all data management (online and offline)
 */

/// Class to manage SDK data.
OBJC_EXPORT @protocol PTRDataManagerInterface<NSObject>

PTR_GENERATE_LISTENER_DECLARATIONS(PTRDataManagerDelegate)

/////////////////////////////////////////////////////////////////////////
#pragma mark - Public methods
/////////////////////////////////////////////////////////////////////////

/**
 * Load data for a given site
 *
 * @param siteId site internal identifier
 */
- (void)loadDataForSite:(int)siteId;

/**
 * Load data for a given site
 *
 * @param siteId site internal identifier
 * @param shouldRespectCachePolicy If YES, it will wait until the cache expires if data is already present. If NO, ignores the internal cache and triggers data update.
 */
- (void)loadDataForSite:(int)siteId shouldRespectCachePolicy:(BOOL)shouldRespectCachePolicy;

/**
 * Blocks and waits until all ongoing data management are done
 */
- (void)waitUntilDone;

/**
 * Returns whether data management (for any site) is currently running
 */
- (BOOL)isRunning;

/**
 * Returns YES if site has all content (e.g. POIs) available to display
 */
- (BOOL)isContentReadyForSite:(int)siteId;

@end

NS_ASSUME_NONNULL_END
