//
//  PTRPoiManagerInterface.h
//  Pointr
//
//  Created by Pointr on 21/03/16.
//  Copyright © 2019 Pointr. All rights reserved.
//

#import <AvailabilityMacros.h>

#import <PointrKit/PTRListenerHelper.h>
#import <PointrKit/PTRPoi.h>
#import <PointrKit/PTRPoiContainer.h>

#import <PointrKit/PTRLevel.h>
#import <PointrKit/PTRSite.h>
#import <PointrKit/PTRBuilding.h>

NS_ASSUME_NONNULL_BEGIN

/**
 *  Delegate to keep track of Poi data updates
 */
OBJC_EXPORT @protocol PTRPoiManagerDelegate<NSObject>

@optional
/**
 *  Notifies when Poi data is changed for a site
 *  Listeners should reload Pois once they hear this callback
 */
- (void)onPoiManagerChangedPoisForSite:(nonnull PTRSite *)site;

@end

/**
 * Main interface to manage accessing Poi content
 */
OBJC_EXPORT @protocol PTRPoiManagerInterface<NSObject>

PTR_GENERATE_LISTENER_DECLARATIONS(PTRPoiManagerDelegate)

/////////////////////////////////////////////////////////////////////////
#pragma mark - Getters
/////////////////////////////////////////////////////////////////////////

/**
 * Checks whether the poi content is present & ready for the given site.
 * It expects the site to have at least one poi to return true.
 *
 * @param site  the site to get the pois for
 * @return BOOL YES if poi content is ready, NO otherwise.
 */
- (BOOL)hasContentForSite:(nonnull PTRSite *)site;

/**
 * Returns all the pois in a given site.
 *
 * @param site the site to get the pois for.
 * @return PTRPoiContainer with all the pois for the site. This method returns nil if the poi data is not available.
 */
- (nullable PTRPoiContainer *)poisForSite:(nonnull PTRSite *)site;

/**
 * Returns all the pois in a given site filtered by types.
 *
 * @param site  the site to get the pois for
 * @param types a list of types to filter the pois. Only pois with types that match one of the types in
 * the list are included. If the list is empty, all pois are included.
 * @return PTRPoiContainer with all the filtered pois for the site. This method returns nil if the poi data is not
 * available.
 */
- (nullable PTRPoiContainer *)poisForSite:(nonnull PTRSite *)site withTypes:(nonnull NSArray<NSString *> *)types;

/**
 * Returns all the pois in a given site filtered by types.
 *
 * @param site  the site to get the pois for
 * @param types a list of types to filter the pois. Only pois with types that match one of the types in
 * the list are included. If the list is empty, all pois are included.
 * @param shouldSortByProximity if this parameter is set to true, then pois are ordered closest first otherwise
 * alphabetical order is used.
 * @return PTRPoiContainer with all the filtered pois for the site. This method returns nil if the poi data is not
 * available.
 */
- (nullable PTRPoiContainer *)poisForSite:(nonnull PTRSite *)site
                                withTypes:(nonnull NSArray<NSString *> *)types
                    shouldSortByProximity:(BOOL)shouldSortByProximity;

/**
 * Returns all the pois in the given building.
 *
 * @param building the building to get pois for
 * @return PTRPoiContainer with all the pois for the building. This method returns nil if the poi data is not available.
 */
- (nullable PTRPoiContainer *)poisForBuilding:(nonnull PTRBuilding *)building;

/**
 * Returns all the pois in a given building filtered by types.
 *
 * @param building the building to get pois for
 * @param types a list of types to filter the pois. Only pois with types that match one of the types in
 * the list are included. If the list is empty, all pois are included.
 * @return PTRPoiContainer with all the filtered pois for the building. This method returns nil if the poi data is not
 * available.
 */
- (nullable PTRPoiContainer *)poisForBuilding:(nonnull PTRBuilding *)building withTypes:(nonnull NSArray<NSString *> *)types;

/**
 * Returns all the pois in a given building filtered by types.
 *
 * @param building the building to get pois for
 * @param types a list of types to filter the pois. Only pois with types that match one of the types in
 * the list are included. If the list is empty, all pois are included.
 * @param shouldSortByProximity if this parameter is set to true, then pois are ordered closest first otherwise
 * alphabetical order is used.
 * @return PTRPoiContainer with all the filtered pois for the building. This method returns nil if the poi data is not
 * available.
 */
- (nullable PTRPoiContainer *)poisForBuilding:(nonnull PTRBuilding *)building
                                    withTypes:(nonnull NSArray<NSString *> *)types
                        shouldSortByProximity:(BOOL)shouldSortByProximity;

/**
 * Returns all the pois in a given level.
 *
 * @param level the level to get pois for
 * @return PTRPoiContainer with all the pois for the level. This method returns nil if the poi data is not available.
 */
- (nullable PTRPoiContainer *)poisForLevel:(nonnull PTRLevel *)level;

/**
 * Returns all the pois in a given level filtered by types.
 *
 * @param level the level to get pois for
 * @param types a list of types to filter the pois. Only pois with types that match one of the types in
 * the list are included. If the list is empty, all pois are included.
 * @return PTRPoiContainer with all the filtered pois for the level. This method returns nil if the poi data is not
 * available.
 */
- (nullable PTRPoiContainer *)poisForLevel:(nonnull PTRLevel *)level withTypes:(nonnull NSArray<NSString *> *)types;

/**
 * Returns all the pois in a given level filtered by types.
 *
 * @param level the level to get pois for
 * @param types a list of types to filter the pois. Only pois with types that match one of the types in
 * the list are included. If the list is empty, all pois are included.
 * @param shouldSortByProximity if this parameter is set to true, then pois are ordered closest first otherwise
 * alphabetical order is used.
 * @return PTRPoiContainer with all the filtered pois for the level. This method returns nil if the poi data is not
 * available.
 */
- (nullable PTRPoiContainer *)poisForLevel:(nonnull PTRLevel *)level
                                 withTypes:(nonnull NSArray<NSString *> *)types
                     shouldSortByProximity:(BOOL)shouldSortByProximity;

/**
 * Returns all the pois in a given site filtered by name.
 * Note there might be multiple Poi objects with the same name.
 *
 * @param site the site to get the pois for
 * @param name name of the poi
 * @return PTRPoiContainer with all the filtered pois for the site. This method returns nil if the poi data is not
 * available.
 */
- (nullable PTRPoiContainer *)poisForSite:(nonnull PTRSite *)site withName:(nonnull NSString *)name;

/**
 * Returns all the pois in a given building filtered by name.
 * Note there might be multiple Poi objects with the same title.
 *
 * @param building the building to get pois for
 * @param name name of the poi
 * @return PTRPoiContainer with all the filtered pois for the building. This method returns nil if the poi data is not
 * available.
 */
- (nullable PTRPoiContainer *)poisForBuilding:(nonnull PTRBuilding *)building withName:(nonnull NSString *)name;

/**
 * Returns all the pois in a given level filtered by name.
 * Note there might be multiple Poi objects with the same title.
 *
 * @param level the level to get pois for
 * @param name name of the poi
 * @return PTRPoiContainer with all the filtered pois for the level. This method returns nil if the poi data is not
 * available.
 */
- (nullable PTRPoiContainer *)poisForLevel:(nonnull PTRLevel *)level withName:(nonnull NSString *)name;

/**
 * Returns a poi from a given site with an identifier.
 *
 * @param site the site to get the poi for
 * @param identifier the identifier of the poi
 * @return PTRPoi with a given identifier. This method returns nil if the poi data is not available.
 */
- (nullable PTRPoi *)poisForSite:(nonnull PTRSite *)site withIdentifier:(nonnull NSString *)identifier;


/**
 * Returns a poi from a given site with an external identifier.
 *
 * @param site the site to get the poi for
 * @param externalIdentifier the external identifier of the poi
 * @return PTRPoi with a given external identifier. This method returns nil if the poi data is not available.
 */
- (nullable PTRPoi *)poisForSite:(nonnull PTRSite *)site withExternalIdentifier:(nonnull NSString *)externalIdentifier;

/////////////////////////////////////////////////////////////////////////
#pragma mark - Search
/////////////////////////////////////////////////////////////////////////

/**
 * Returns PTRPoiContainer that match the given search string.
 * Search is done on single word typically and is case insensitive. If more than one word is supplied, an OR search is
 * performed using every word. Sorted alphabetically ignoring case by title.
 *
 * @param site the site to search in
 * @param searchString the query string
 * @return PTRPoiContainer that matches the query ordered by relevance
 */
- (nonnull PTRPoiContainer *)searchPoisInSite:(nonnull PTRSite *)site withString:(nonnull NSString *)searchString;

/**
 * Returns PTRPoiContainer that match the given search string filtered by types.
 * Search is done on single word typically and is case insensitive. If more than one word is supplied, an OR search is
 * performed using every word. Sorted alphabetically ignoring case by title.
 *
 * @param site the site to search in
 * @param searchString the query string
 * @param types  only search for pois with these types
 * @return PTRPoiContainer that matches the query and types ordered by relevance
 */
- (nonnull PTRPoiContainer *)searchPoisInSite:(nonnull PTRSite *)site
                                   withString:(nonnull NSString *)searchString
                                     andTypes:(nonnull NSArray<NSString *> *)types;

/**
 * Returns PTRPoiContainer that match the given search string.
 * Search is done on single word typically and is case insensitive. If more than one word is supplied, an OR search is
 * performed using every word. Sorted alphabetically ignoring case by title.
 *
 * @param building the building to search in
 * @param searchString the query string
 * @return PTRPoiContainer that matches the query ordered by relevance
 */
- (nonnull PTRPoiContainer *)searchPoiInBuilding:(nonnull PTRBuilding *)building withString:(nonnull NSString *)searchString;

/**
 * Returns PTRPoiContainer that match the given search string.
 * Search is done on single word typically and is case insensitive. If more than one word is supplied, an OR search is
 * performed using every word. Sorted alphabetically ignoring case by title.
 *
 * @param building the building to search in
 * @param searchString the query string
 * @param types  only search for pois with this types
 * @return PTRPoiContainer that matches the query ordered by relevance
 */
- (nonnull PTRPoiContainer *)searchPoiInBuilding:(nonnull PTRBuilding *)building
                                      withString:(nonnull NSString *)searchString
                                        andTypes:(nonnull NSArray<NSString *> *)types;

/////////////////////////////////////////////////////////////////////////
#pragma mark - Poi Type
/////////////////////////////////////////////////////////////////////////

/// Method to get all the poi types available on a specific site.
/// @param site The site
/// @return Array of types in string format.
- (nonnull NSArray<NSString *> *)poiTypesForSite:(nonnull PTRSite *)site;

@end

NS_ASSUME_NONNULL_END
