//
// Created by Pointr Labs on 22/07/16.
// Copyright (c) 2016 Pointr. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>

/**
 * Pointr category for UIView class.
 */
OBJC_EXPORT @interface UIView(Pointr)

/// Setter and getter for self.frame.origin.x
@property float x;

/// Setter and getter for self.frame.origin.y
@property float y;

/// Setter and getter for self.frame.size.width
@property float width;

/// Setter and getter for self.frame.size.height
@property float height;

/**
 * Method to add top, leading, bottom and trailing constraints based on UIEdgeInsets.
 *
 * @param insets   Insets that determine the constraints constants.
 * @param toMargin Pass YES if you want the constraint to the safe margin, NO if to the edge.
 *
 * @return Array of NSLayoutConstraint objects added.
 */
- (nonnull NSArray<NSLayoutConstraint *> *)ptr_addContraintsToSuperviewWithInsets:(UIEdgeInsets)insets
                                                                         toMargin:(BOOL)toMargin;

/**
 * Method to add top, leading, bottom and trailing constraints.
 *
 * @param top Top distance.
 * @param bottom Bottom distance.
 * @param leading Leading distance.
 * @param trailing Trailing distance.
 * @param toMargin Pass YES if you want the constraint to the safe margin, NO if to the edge.
 * @return Array of NSLayoutConstraint objects added.
 */
- (nonnull NSArray<NSLayoutConstraint *> *)ptr_addContraintsToSuperViewWithTop:(nullable NSNumber *)top
                                                                        bottom:(nullable NSNumber *)bottom
                                                                       leading:(nullable NSNumber *)leading
                                                                      trailing:(nullable NSNumber *)trailing
                                                                      toMargin:(BOOL)toMargin;

/**
 * Method to add a top constraint to the superview.
 *
 * @param constant Constraint's constant value.
 * @param toMargin Pass YES if you want the constraint to the safe margin, NO if to the edge.
 * @return NSLayoutConstraint object added.
 */
- (nullable NSLayoutConstraint *)ptr_addTopConstraintToSuperviewWithConstant:(CGFloat)constant toMargin:(BOOL)toMargin;

/**
 * Method to add a bottom constraint to the superview.
 *
 * @param constant Constraint's constant value.
 * @param toMargin Pass YES if you want the constraint to the safe margin, NO if to the edge.
 * @return NSLayoutConstraint object added.
 */
- (nullable NSLayoutConstraint *)ptr_addBottomConstraintToSuperviewWithConstant:(CGFloat)constant
                                                                       toMargin:(BOOL)toMargin;

/**
 * Method to add a leading constraint to the superview.
 *
 * @param constant Constraint's constant value.
 * @param toMargin Pass YES if you want the constraint to the safe margin, NO if to the edge.
 * @return NSLayoutConstraint object added.
 */
- (nullable NSLayoutConstraint *)ptr_addLeadingConstraintToSuperviewWithConstant:(CGFloat)constant
                                                                        toMargin:(BOOL)toMargin;

/**
 * Method to add a trailing constraint to the superview.
 *
 * @param constant Constraint's constant value.
 * @param toMargin Pass YES if you want the constraint to the safe margin, NO if to the edge.
 * @return NSLayoutConstraint object added.
 */
- (nullable NSLayoutConstraint *)ptr_addTrailingConstraintToSuperviewWithConstant:(CGFloat)constant
                                                                         toMargin:(BOOL)toMargin;

/**
 * Method to add a top constraint to relate with another view. By default, it will relate with the opposite attribute of
 * the other view.
 *
 * @param view View to related with.
 * @param constant Constraint's constant value.
 * @return NSLayoutConstraint object added.
 */
- (nullable NSLayoutConstraint *)ptr_addTopConstraintToView:(nonnull UIView *)view constant:(CGFloat)constant;

/**
 * Method to add a bottom constraint to relate with another view. By default, it will relate with the opposite attribute
 * of the other view.
 *
 * @param view View to related with.
 * @param constant Constraint's constant value.
 * @return NSLayoutConstraint object added.
 */
- (nullable NSLayoutConstraint *)ptr_addBottomConstraintToView:(nonnull UIView *)view constant:(CGFloat)constant;

/**
 * Method to add a leading constraint to relate with another view. By default, it will relate with the opposite
 * attribute of the other view.
 *
 * @param view View to related with.
 * @param constant Constraint's constant value.
 * @return NSLayoutConstraint object added.
 */
- (nullable NSLayoutConstraint *)ptr_addLeadingConstraintToView:(nonnull UIView *)view constant:(CGFloat)constant;

/**
 * Method to add a trailing constraint to relate with another view. By default, it will relate with the opposite
 * attribute of the other view.
 *
 * @param view View to related with.
 * @param constant Constraint's constant value.
 * @return NSLayoutConstraint object added.
 */
- (nullable NSLayoutConstraint *)ptr_addTrailingConstraintToView:(nonnull UIView *)view constant:(CGFloat)constant;

/**
 * Method to add a top constraint to relate with another view.
 *
 * @param view View to related with.
 * @param attribute Other view attribute.
 * @param constant Constraint's constant value.
 * @return NSLayoutConstraint object added.
 */
- (nullable NSLayoutConstraint *)ptr_addTopConstraintToView:(nonnull UIView *)view
                                                  attribute:(NSLayoutAttribute)attribute
                                                   constant:(CGFloat)constant;

/**
 * Method to add a bottom constraint to relate with another view.
 *
 * @param view View to related with.
 * @param attribute Other view attribute.
 * @param constant Constraint's constant value.
 * @return NSLayoutConstraint object added.
 */
- (nullable NSLayoutConstraint *)ptr_addBottomConstraintToView:(nonnull UIView *)view
                                                     attribute:(NSLayoutAttribute)attribute
                                                      constant:(CGFloat)constant;

/**
 * Method to add a leading constraint to relate with another view.
 *
 * @param view View to related with.
 * @param attribute Other view attribute.
 * @param constant Constraint's constant value.
 * @return NSLayoutConstraint object added.
 */
- (nullable NSLayoutConstraint *)ptr_addLeadingConstraintToView:(nonnull UIView *)view
                                                      attribute:(NSLayoutAttribute)attribute
                                                       constant:(CGFloat)constant;

/**
 * Method to add a trailing constraint to relate with another view.
 *
 * @param view View to related with.
 * @param attribute Other view attribute.
 * @param constant Constraint's constant value.
 * @return NSLayoutConstraint object added.
 */
- (nullable NSLayoutConstraint *)ptr_addTrailingConstraintToView:(nonnull UIView *)view
                                                       attribute:(NSLayoutAttribute)attribute
                                                        constant:(CGFloat)constant;

/**
 * Method to add a width constraint.
 *
 * @param constant Constraint's constant value.
 * @return NSLayoutConstraint object added.
 */
- (nonnull NSLayoutConstraint *)ptr_addWidthConstraintWithConstant:(CGFloat)constant;

/**
 * Method to add a height constraint.
 *
 * @param constant Constraint's constant value.
 * @return NSLayoutConstraint object added.
 */
- (nonnull NSLayoutConstraint *)ptr_addHeightConstraintWithConstant:(CGFloat)constant;

/**
 * Method to center the view vertically in it's superview.
 *
 * @param constant Constraint's constant value.
 * @return NSLayoutConstraint object added.
 */
- (nullable NSLayoutConstraint *)ptr_addCenterVerticalConstraintWithConstant:(CGFloat)constant;

/**
 * Method to center the view horizontally in it's superview.
 *
 * @param constant Constraint's constant value.
 * @return NSLayoutConstraint object added.
 */
- (nullable NSLayoutConstraint *)ptr_addCenterHorizontalConstraintWithConstant:(CGFloat)constant;

@end
